<?php

class MailboxModule extends CWebModule
{
	
	/**
	* @property string the name of the user model class.
	*/
	public static $userClass = 'User';
	/**
	* @property string the name of the id column in the user table.
	*/
	public static $userIdColumn ='id';
	/**
	* @property string the name of the username column in the user table.
	*/
	public static $usernameColumn = 'username';
	/**
	* @property string the name of the column in your User model that defines whether user is an admin.
	*/
	public static $superuserColumn = 'superuser';
	/**
	* @property string text to display if user not found.
	*/
	public $deletedUser = 'user deleted';
	/**
	* @property integer number of conversations to display per page.
	*/
	public $pageSize = 10;
	/**
	* @property mixed if you want to use an authManager (such as the Rights module) to control the access rules
	* then set to true or if your using the Rights module set to 'rights' in order to include the Rights filter.
	*/
	public $authManager = false; // 'rights'
	/**
	 *
	 * @property boolean enable a read-only mailbox for users. Admins will still have fully functional mailbox. 
	 */
	public $readOnly = false;
	/**
	 *
	 * @property boolean whether to allow user's to message other users. Ie false means users can only contact admins 
	 */
	public $userToUser = true;
	
	/*
	 * The following properties do not have an affect if you are using an authManager such as Rights. In this case the authManager will control the access.
	 *	- sendMsgs
	 *	- sentbox
	 *	- trashbox
	 */
	
	/**
	* @property boolean whether or not to allow users to send messages.
	*/
	public $sendMsgs = true;
	/**
	* @property boolean whether or not to enable/disable sent folder for viewing sent messages.
	*/
	public $sentbox = true;
	/**
	* @property boolean whether or not to enable/disable trash folder for recycling deleted messages. 
	 * If trashbox is enabled then you must run the cron() method at least once daily, ideally each morning.
	*/
	public static $trashbox = true;
	/**
	* @property integer number of days to keep deleted items in trash folder before being permanently deleted by the cron() method.
	*/
	public static $recyclePeriod = 7;
	/**
	* @property boolean whether to allow users, when viewing the sent folder, to see if recipient has read the sent message.
	*/
	public $recipientRead = true;
	/**
	 * @property string|boolean set to custom css file location relative to base URL or set to false to skip css includes
	 */
	public $cssFile;
	/**
	 * @property string set to either 'left' or 'top' to position the mailbox menu. If readOnly set to true this is ignored for user and only applied for admins (since there is no menu in readOnly mode). 
	 */
	public $menuPosition = 'left';
	/**
	 * @property string|boolean custom left menu css
	 */
	public $cssFileColumn;
	/**
	 * @property string default subject to use when no subject is provided.
	 */
	public $defaultSubject = '(no subject)';
	/**
	 * @property string list or allowable characters to check Subject field against 
	 * using case-insensitive regular expression. Square brackets '[]', dashes '-', 
	 * single quotes, and slashes '/\' need to be escaped in order to be used literally.
	 */
	public $allowableCharsSubject = '0-9a-z.,!?@\s*$%#&;:+=_(){}\[\]\/\\-';
	/**
	 * @property array HTML Purify options array used to purify HTML input from admins. 
	 * See http://htmlpurifier.org/live/configdoc/plain.html for list of options.
	 * Set to an empty string to strip all HTML tags from message input.
	*/
	public $adminHtml = array('HTML.Allowed'=>'a[href],b,strong,p,u,i,hr,br,img[src|alt|border]');
	/**
	 * @property array HTML Purify options array used to purify HTML input from non-admins. 
	 * See http://htmlpurifier.org/live/configdoc/plain.html for list of options. 
	 * Set to an empty string to strip all HTML tags from message input.
	*/
	public $html = array('HTML.Allowed'=>'a[href],b,strong,p,u,i,hr,br');
	/**
	* @property boolean whether to allow users to be able to enter user Id's instead of username when sending messages.
	*/
	public $allowLookupById = true;
	/**
	* @property boolean whether to create a link for the From field to user's profile etc (link created by getUrl() method). This attribute is always true for admins unless the getUrl() method returns false.
	*/
	public $linkUser = true;
	/**
	 * @property boolean whether to use checksums when storing messages. Checksums can be used not only for validating data but also to help implement certain spam protection.
	 * I.e. if a user copy and paste a message and sends it to multiple users you can easily find these messages by searching for other messages with the same checksum. 
	 */
	public $checksums = 1;
	/**
	 * TODO: not yet implemented. Please ignore.
	 * @property integer whether to validate the checksum when reading a message. If validation fails an error message will show. Set to 0 to ignore this feature, you can still use checksums for spam blocking.
	 *	0 - Do not validate (recommended)
	 *	1 - Only validate for admins
	 *	2 - Validate for all
	*/
	public $validateChecksum = 0;
	/**
	* @property boolean whether to enable debug mode.
	*/
	public $debug = true;
	
	public $defaultController = 'message';
	private $_assetsUrl;
	private $_new;
	
	public function init()
	{
		// import the module-level models and components
		$this->setImport(array(
			'mailbox.models.*',
			'mailbox.components.*',
		));
		
		$this->registerScripts();
		
	}
	
	public static function getUserId($username='')
	{
		if($username)
		{
			return call_user_func(array(MailboxModule::$userClass, 'model'))->findByAttributes(array(MailboxModule::$usernameColumn=>$username))->{MailboxModule::$userIdColumn};
		}
		else	
		{
			return  Yii::app()->user->{MailboxModule::$userIdColumn};
		}
	}
	
	public static function getUserName($userid='')
	{
		//return 'demo';
		if(!$userid)
			$userid = Yii::app()->user->{MailboxModule::$userIdColumn};
		
		return call_user_func(array(MailboxModule::$userClass, 'model'))->findByPk($userid)->{MailboxModule::$usernameColumn};
		
		//return User::model()->getUserNameById($userid)->find()->$this->usernameColumn;
	}
	
	public static function getFromLabel($userid)
	{
		return self::getUserName($userid);
	}
	
	public static function getUrl($userid)
	{
		//return null;
		return Yii::app()->createUrl('user', array('user' => $userid));
	}
	
	public static function isAdmin($userid='')
	{
		if(!$userid)
			$userid = self::getUserId();
		
		return call_user_func(array(MailboxModule::$userClass, 'model'))->findByPk($userid)->{MailboxModule::$superuserColumn};
		
		//return User::model()->findbyPk($userid)->superuser;
	}
	
	public static function autoComplete($term)
	{
		
		$criteria = new CDbCriteria;

		$criteria->compare(MailboxModule::$usernameColumn, $term, true, 'OR');
		$criteria->compare(MailboxModule::$userIdColumn, $term, true, 'OR');
		$criteria->mergeWith(array('limit'=>25));
		$users = call_user_func(array(MailboxModule::$userClass, 'model'))
			->findAll($criteria);
			//->findAllByAttributes(array(MailboxModule::$usernameColumn=>$term) , array('limit'=>'25'));
		//$users = User::model()->keyword($term)->limit(100)->findAll();
		$json = '[';
		foreach($users as $user)
		{
			$json .= '"'.$user->{MailboxModule::$usernameColumn}.'",';
		}
		$json = rtrim($json,',') . ']';
		die($json);
	}
	
	/**
	* Registers the necessary scripts.
	*/
	public function registerScripts()
	{
		// Get the url to the module assets
		$assetsUrl = $this->getAssetsUrl();
		//Yii::app()->getAssetManager()->publish('assets');
		// Register the necessary scripts
		$cs = Yii::app()->getClientScript();
		$cs->registerCoreScript('jquery');
		$cs->registerCoreScript('jquery.ui');
		$cs->registerScriptFile($assetsUrl.'/js/mailbox.js');
		$cs->registerScriptFile($assetsUrl.'/js/jquery.textTruncate.js');
		$cssCoreUrl = $cs->getCoreScriptUrl();
		$cs->registerCssFile($cssCoreUrl . '/jui/css/base/jquery-ui.css'); 
		//$cs->registerCssFile($assetsUrl.'/css/mailbox.css');
		
		if( $this->cssFile!==false )
		{
			// Default style sheet is used unless one is provided.
			if( $this->cssFile===null )
				$this->cssFile = $assetsUrl.'/css/mailbox.css';
			else
				$this->cssFile = Yii::app()->request->baseUrl.$this->cssFile;
			
			if( $this->menuPosition == 'left' && $this->cssFileColumn===null  && !$this->readOnly)
				$this->cssFileColumn = $assetsUrl.'/css/mailbox_column.css';
			elseif($this->menuPosition == 'left' && !$this->readOnly)
				$this->cssFileColumn = Yii::app()->request->baseUrl.$this->cssFileColumn;

				
			// Register the style sheet
			$cs->registerCssFile($this->cssFile);
			$cs->registerCssFile($this->cssFileColumn);
		}
	}
	
	
	/**
	* Publishes the module assets path.
	* @return string the base URL that contains all published asset files of Rights.
	*/
	public function getAssetsUrl()
	{
		if( $this->_assetsUrl===null )
		{
			$assetsPath = Yii::getPathOfAlias('mailbox.assets');

			// We need to republish the assets if debug mode is enabled.
			if( $this->debug===true )
				$this->_assetsUrl = Yii::app()->getAssetManager()->publish($assetsPath, false, -1, true);
			else
				$this->_assetsUrl = Yii::app()->getAssetManager()->publish($assetsPath);
		}

		return $this->_assetsUrl;
	}

	public function beforeControllerAction($controller, $action)
	{
		if (Yii::app()->user->isGuest) {
			if (Yii::app()->user->loginUrl) {
				$controller->redirect($controller->createUrl(reset(Yii::app()->user->loginUrl)));
			} else {
				$controller->redirect($controller->createUrl('/'));
			}
		} else if (parent::beforeControllerAction($controller, $action)) {
			// this method is called before any module controller action is performed
			// you may place customized code here
			return true;
		} else {
			return false;
		}
	}

	public static function t($str='',$params=array(),$dic='message') {
		return Yii::t("MessageModule.".$dic, $str, $params);
	}
	
	public static function getNewMsgs()
	{
		return Mailbox::newMsgs();
		// Update message count every 5 min
		if(!$_SESSION['Mailbox']['new'] || $_SESSION['Mailbox']['timestamp'] < (time() + 300) )
		{
			$_SESSION['Mailbox']['new'] = Mailbox::newMsgs();
			$_SESSION['Mailbox']['timestamp'] = time();
		}
		return $_SESSION['Mailbox']['new'];
	}
	
	public static function cron()
	{
		$deleted_convs = Mailbox::model()->getDeleted()->findAll();
		
		foreach($deleted_convs as $conv)
		{
			$conv->recycle();
		}
	}
}
